@@ -32,7 +32,7 @@ class Fs_cdr_config_class extends Safe_configurable_object_class
      */
     public function __construct($node, $parent_name)
     {
-        parent::__construct($parent_name, "cdr");
+        parent::__construct($parent_name, "configuration");
     }
     /**
      * @brief Configure the config class object
@@ -56,7 +56,6 @@ class Fs_cdr_config_class extends Safe_configurable_object_class
         $this->conditional_control('global/local', 'local');
         
         // local storage hidden fields
-        $this->add_field('local/log-dir', 'log-dir', 'hidden', '');
         $this->add_field('local/max_size', 'Local Max Size', 'int', '100', 5);
         $this->set_field_rules('local/max_size', 'required[global/local]|integer|greater_than[0]');
         $this->set_field_help('local/max_size',
@@ -98,13 +97,6 @@ class Fs_cdr_config_class extends Safe_configurable_object_class
         //$this->set_field_help('http/retries', 'HELP for http/retries!');
         $this->add_field('http/delay', 'Retry Delay', 'int', 5);
         $this->set_field_rules('http/delay', 'required[global/http]|greater_than[0]');
-        // HTTP storage hidden fields
-        $this->add_field('http/encode', 'encode', 'hidden', 'textxml');
-        $this->add_field('http/log-http-and-disk', 'log-http-and-disk', 'hidden', 'true');
-        $this->add_field('http/auth-scheme', 'auth-scheme', 'hidden', 'any');
-        // common stuff (hidden)
-        $this->add_field('common/log-b-leg', 'log-b-leg', 'hidden', 'true');
-        $this->add_field('common/prefix-a-leg', 'prefix-a-leg', 'hidden', 'false');
         
         return parent::configure();
     }
@@ -154,7 +146,7 @@ class Fs_cdr_module_class extends Fs_module_class
         // Set the module description
         $this->set_description("CDR");
         // Create the config object
-        $cfg = new Fs_cdr_config_class($this->node() , $this->fs_app()->object_name() , $this->name());
+        $cfg = new Fs_cdr_config_class($this->node() , $this->object_name() , $this->name());
         // Synch with DB
         $cfg->configure();
         $cfg->synch();
@@ -191,6 +183,20 @@ class Fs_cdr_module_class extends Fs_module_class
     {
         // Get our configuration data (unresolved)
         $cfg = $this->config()->get_data_values_exploded(false);
+
+        // Local storage hidden fields
+        $data_settings = $this->get_data_settings();
+        $xml_dir = $data_settings['cdr']['file_dir'];
+        $cfg['local']['log-dir'] = $xml_dir;
+        // HTTP( storage hidden fields
+        $cfg['http']['encode'] = 'textxml';
+        if ('true' == $cfg['global']['local'] && 'true' == $cfg['global']['http']) {
+            $cfg['http']['log-http-and-disk'] = 'true';
+        }
+        $cfg['http']['auth-scheme'] = 'any';
+        // common stuff (hidden)
+        $cfg['common']['log-b-leg'] = 'true';
+        $cfg['common']['prefix-a-leg'] = 'false';
         
         // Check if local enabled
         if ('true' != $cfg['global']['local']) {
@@ -246,8 +252,6 @@ class Fs_cdr_module_class extends Fs_module_class
         
         //creates a file named sng-cdr-cleanup will be created in /etc/cron.<local/schedule> directory 
         if($cleanup){
-            $data_settings = $this->get_data_settings();
-            $xml_dir = $data_settings['cdr']['file_dir'];
             $file_content = '#!/bin/bash'.PHP_EOL.
             'nice -n 20 /usr/local/nsc/bin/python /usr/local/sng/scripts/sng-cdr-rotator --path ' . $xml_dir . ' --size '.
             $cleanup['max_size'] . ' --service_name ' . $this->fs_app()->name() . PHP_EOL;
@@ -269,9 +273,9 @@ class Fs_cdr_module_class extends Fs_module_class
             'cdr' => array(
                 'description' => 'Call Data Records',
                 'filesystem' => true,
-                'pattern' => $cfg_manager->directory('log') . '/xml_cdr/*.xml',
+                'pattern' => $cfg_manager->directory('data','') . '/xml_cdr/*.xml',
                 'file_ext' => 'cdr.xml',
-                'file_dir' => $cfg_manager->directory('log') . '/xml_cdr',
+                'file_dir' => $cfg_manager->directory('data','') . '/xml_cdr',
                 'recursive' => true,
                 'global_capabilities' => array(
                     'download',
@@ -281,7 +285,19 @@ class Fs_cdr_module_class extends Fs_module_class
                     'download',
                     'delete',
                     'view'
-                )
+                ),
+                'methods' => array(
+                        'download' => array(
+                                'name' => 'Download',
+                                'description' => array( 'Download cdr files.' ),
+                                'request' => 'GET',
+                        ),
+                        'delete' => array(
+                                'name' => 'Delete',
+                                'description' => array( 'Delete cdr files.' ),
+                                'request' => 'POST',
+                        ),
+                ),
             )
         );
         return $data;
@@ -294,6 +310,13 @@ class Fs_cdr_module_class extends Fs_module_class
         preg_match($regex_pattern, $base, $match);
         return $match['key'];
     }
+    public function api_download_cdr($name=null, $data = null, &$output = null){
+        $this->download_data('cdr', $name);
+        exit();
+    }
+    public function api_delete_cdr($name=null, $data = null, &$output = null){
+        return $this->delete_data('cdr', $name);
+    }
     /**
      * @brief Invoked after a successfull write_config
      *         
