@@ -194,8 +194,10 @@ class Safe_database_class
     public function insert($table, $data)
     {
         //Checks to see if the command ran successfully
-        if ($this->db->insert($table, $data)) if ($this->db->affected_rows() > 0) return true;
-        else return false;
+        if ($this->db->insert($table, $data)) {
+            if ($this->db->affected_rows() > 0) return true;
+            else return false;
+        }
         else return false;
     }
     /**
@@ -210,7 +212,9 @@ class Safe_database_class
      */
     public function delete($table, $pk, $data)
     {
-        foreach ($pk as $key) $this->db->where($key, $data[$key]);
+        foreach ($pk as $key){
+            $this->db->where($key, $data[$key]);
+        }
         //Checks to see if the command ran successfully
         if ($this->db->delete($table)) return true;
         else return false;
@@ -238,14 +242,53 @@ class Safe_database_class
         if ($this->db->update($table, $data)) return true;
         else return false;
     }
+
+    public function load_data($table, $data = array())
+    {
+        // Get temp file
+        $tmpfname = $data['file_name'];
+        $keys = $data['keys'];
+
+        // make sure mysql can read it
+        chmod($tmpfname, 0644);
+        $query_str = 
+            "LOAD DATA INFILE '" . $tmpfname . "' INTO TABLE " . $table ." FIELDS TERMINATED BY ',' ".
+            " (".implode(",", $keys) . ");";
+        $query = $this->db->query($query_str);
+
+        if($query){
+            $rc = true;
+        }else{
+            $rc = false;
+        }
+        @unlink($tmpfname);
+        return $rc;
+    }
+    public function query($query_str)
+    {
+        $query = $this->db->query($query_str);
+
+        if($query){
+            $rc = true;
+        }else{
+            $rc = false;
+        }
+        return $rc;
+    }
 }
 class Safe_mysql_database_class extends Safe_database_class
 {
     public function configure($config)
     {
         //Sets codeignitor active record
-        if ($config['active-record']) $active_record = true;
-        else $active_record = false;
+        if ($config['active-record']){
+            $active_record = true;
+        }else{
+            $active_record = false;
+        }
+        // Force disabling db_debug
+        $config['db_debug'] = false;
+        // Configure
         return parent::configure($config);
     }
 }
