@@ -303,7 +303,7 @@ class Safe_form_class
                 if ($v['type'] == 'hidden') {
                     isset($v['value']) && $this->hiddens[$v['field']] = $v['value'];
                     unset($this->config[$k]); // not necessary
-                }elseif($v['type'] == 'private'){
+                }elseif($v['type'] == 'private' || $v['type'] == 'base64'){
                     isset($v['value']) && $this->privates[$v['field']] = $v['value'];
                     unset($this->config[$k]); // not necessary
                 }else {
@@ -345,7 +345,11 @@ class Safe_form_class
         // return
         // form_open($this->action,$this->form_attributes,$this->hiddens)."\n";
         // i trust the user so the uri is not rewritten by codigniter...
-        $html = form_open('__://__', $this->form_attributes, $this->hiddens + $this->privates) . "\n";
+        $_attr = $this->form_attributes;
+        if(!array_key_exists("enctype", $attr)){
+            $_attr["enctype"] = "multipart/form-data";
+        }
+        $html = form_open('__://__', $_attr, $this->hiddens + $this->privates) . "\n";
         $html = preg_replace('#__://__#', $this->action, $html);
         return $html;
     }
@@ -466,9 +470,9 @@ class Safe_form_class
                 break;
 
             case 'hidden':
-                break;
             case 'private':
-                    break;
+            case 'base64': // base64 are not meant to be displayed
+                break;
 
             default:
                 $temp_conf['value'] = set_value($curr_conf['field'], (isset($curr_conf['default'])) ? $curr_conf['default'] : null);
@@ -570,9 +574,9 @@ class Safe_form_class
                 }
                 return $_select_html;
             case 'hidden':
-                break;
             case 'private':
-                    break;
+            case 'base64':
+                break;
 
             case 'date':
             case 'datetime':
