@@ -18,8 +18,13 @@
  * OS class
 */
 require_once ('application/helpers/safe_helper.php');
-class Safe_os_wrapper
+safe_require_class('safe_auditor_class');
+class Safe_os_wrapper extends Safe_object_class
 {
+    public function __construct()
+    {
+        parent::__construct('', 'os');
+    }
     public function execute($cmd, &$result = null, $escape = true, $superuser = false, &$return_var=null, $log_error=true)
     {
         // PHP handle the os specific exec stuff
@@ -38,39 +43,60 @@ class Safe_os_wrapper
     }
     public function mkdir($pathname, $chown_recursive=true)
     {
-        if (!file_exists($pathname)) Safe_sudo_exec('/bin/mkdir -p ' . $pathname);
+        if (!file_exists($pathname)){
+            Safe_sudo_exec('/bin/mkdir -p ' . $pathname);
+            clearstatcache(true, $pathname);
+        }
         if($chown_recursive == true){
             Safe_sudo_exec('/bin/chown -R webconfig ' . $pathname);
         }else{
-            Safe_sudo_exec('/bin/chown webconfig ' . $pathname);
+            // Check if root is not file owner (test with 0 being root uid
+            if(!fileowner($pathname)) {
+                Safe_sudo_exec('/bin/chown webconfig ' . $pathname);
+            }
         }
         return file_exists($pathname);
     }
     public function rmdir($pathname)
     {
-        if (file_exists($pathname)) Safe_sudo_exec('/bin/rm -rf ' . $pathname);
+        if (file_exists($pathname)){
+            Safe_sudo_exec('/bin/rm -rf ' . $pathname);
+            clearstatcache(true, $pathname);
+        }
         return (!file_exists($pathname));
     }
     public function delete_file($file_name)
     {
-        if (file_exists($file_name)) Safe_sudo_exec('/bin/rm -fr ' . $file_name);
+        if (file_exists($file_name)){
+            Safe_sudo_exec('/bin/rm -fr ' . $file_name);
+            clearstatcache(true, $file_name);
+        }
         return (!file_exists($file_name));
     }
     public function move_file($src_file, $dst_file)
     {
-        if (file_exists($src_file)) Safe_sudo_exec('/bin/mv -f ' . $src_file . ' ' . $dst_file);
+        if (file_exists($src_file)){
+            Safe_sudo_exec('/bin/chown webconfig ' . $src_file);
+            Safe_sudo_exec('/bin/mv -f ' . $src_file . ' ' . $dst_file);
+            clearstatcache(true, $src_file);
+            clearstatcache(true, $dst_file);
+        }
         return (file_exists($dst_file));
     }
     public function copy_file($src_file, $dst_file)
     {
-        if (file_exists($src_file)) Safe_sudo_exec('/bin/cp -f ' . $src_file . ' ' . $dst_file);
+        if (file_exists($src_file)){
+            Safe_sudo_exec('/bin/cp -f ' . $src_file . ' ' . $dst_file);
+            clearstatcache(true, $src_file);
+            clearstatcache(true, $dst_file);
+        }
         return (file_exists($dst_file));
     }
 
     private function _recursiveGlob($dir, $ext) {
         $globFiles = array();
         if(isset($dir) && strlen($dir) && '/' !== $dir){
-            $globFiles = glob("$dir/$ext");
+            $globFiles = glob("$dir/$ext", GLOB_BRACE);
             $globDirs  = glob("$dir/*", GLOB_ONLYDIR);
             foreach ($globDirs as $rdir) {
                 $files = $this->_recursiveGlob($rdir, $ext);
@@ -81,16 +107,19 @@ class Safe_os_wrapper
         return $globFiles;
     }
     
-    public function search_files($pattern, $recursive=false)
+    public function search_files($pattern, $recursive=false, $sort=true)
     {
         if($recursive){
             $dir = dirname($pattern);
             $base = basename($pattern);
             $files = $this->_recursiveGlob($dir, $base);
         }else{
-            $files = glob($pattern);
+            $files = glob($pattern, GLOB_BRACE);
             if (false == $files) $files = array();
         }
+        if($sort){
+            array_multisort(array_map('filemtime', $files), SORT_NUMERIC, SORT_DESC, $files);
+        }
         return $files;
     }
     public function check_file_exists($file_name, $recursive=false)
@@ -109,7 +138,7 @@ class Safe_os_wrapper
         if($recursive){
             $files = $this->_recursiveGlob($dir, $file_name);
         }else{
-            $files = glob("$dir/$file_name");
+            $files = glob("$dir/$file_name", GLOB_BRACE);
         }
         if(isset($files[0])){
             return $files[0];
@@ -151,7 +180,10 @@ class Safe_os_wrapper
      */
     public function unlink_file($file)
     {
-        if (file_exists($file)) Safe_sudo_exec('/bin/rm -f ' . $file);
+        if (file_exists($file)){
+            Safe_sudo_exec('/bin/rm -f ' . $file);
+            clearstatcache(true, $file);
+        }
         return true;
     }
     public function write_file($file_name, $buffer, $mode = 'w')
@@ -291,15 +323,18 @@ class Safe_os_wrapper
                 foreach($output as $matched_string){
                     if($ip != 'ALL'){
                         //check if ip match or not
-                        if(strpos($matched_string,$ip.':'.$port) === false){
-                            $port_used = false;
+                        if(strpos($matched_string,':::'.$port) !== false || strpos($matched_string,'0.0.0.0:'.$port) !== false){
+                        }else{
+                            if(strpos($matched_string,$ip.':'.$port) === false){                                                                                                             
+                                $port_used = false;
+                            }
                         }
                     }
                     $user_prog = substr(strrchr($matched_string, '/' ), 1 );
-                    if($port_used == true && $program_name != '' && false !== ($tmp_pos=strpos($matched_string,$program_name)) ){
-                        if($user_prog == $program_name || $user_prog== 'sh'){
-                            $port_used = false;
-                        }
+                    if($port_used == true && $program_name != '' 
+                        && ($user_prog == $program_name || $user_prog== 'sh') )
+                    {
+                        $port_used = false;
                     }
                     if($port_used) {
                         error_log('valid_ip_port('.$ip.','.$port.','.$type_str.','.$program_name.') used by <'.$matched_string.'>');
@@ -320,34 +355,65 @@ class Safe_os_wrapper
 
         return !$port_used;
     }
+    private $_hostname=null;
+    public function hostname()
+    {
+        if(!$this->_hostname){
+            $hostname[] = "localhost";
+            $this->execute('hostname', $hostname);
+            $this->_hostname = $hostname[0];
+        }
+        return $this->_hostname;
+    }
+    public function statistics()
+    {
+        return array();
+    }
+    public function version()
+    {
+        return false;
+    }
+    public function can_install_data($info, &$reason)
+    {
+        return true;
+    }
 }
 class Safe_centos_wrapper extends Safe_os_wrapper
 {
     public function service_ctl($service_name, $action, &$result = null, $process_name = null)
     {
         $return_var = 0;
+        if (isset($process_name)) $service_name = $process_name;
         switch ($action) {
         case 'status':
             safe_sudo_exec('service', $service_name . ' ' . $action, '', $return_var, false);
             if (!isset($result)) break;
 
         case 'stats':
-            if (isset($process_name)) $service_name = $process_name;
+            /*
             $result['process'] = '';
-            $result['cputime'] = '';
-            $result['%cpu'] = '';
-            $result['%mem'] = '';
-            $output = safe_exec('ps --no-headers -eo comm,etime,%cpu,%mem| grep -w ' . $service_name, '', '', $dummy, false);
-            if (!$dummy) {
-                if (empty($output)) $return_var = - 1;
-                else {
-                    $output[0] = str_replace('  ', ' ', $output[0]);
-                    $stats = explode(' ', trim(str_replace($service_name, ' ', $output[0])));
-                    $result['process'] = $service_name;
-                    $result['cputime'] = $stats[0];
-                    $result['%cpu'] = $stats[1];
-                    $result['%mem'] = $stats[2];
+            $result['cpu']['time'] = '';
+            $result['cpu']['percent'] = '';
+            $result['mem']['percent'] = '';
+            $result['mem']['kb'] = '';
+             */
+            $output = safe_exec('ps --no-headers -eo comm,etime,%cpu,%mem,rss| grep -e "^' . $service_name . '[ \\t]\\+[0-9]"', '', '', $dummy, false);
+            if(!$dummy){
+                if(empty($output)){
+                    $return_var = - 1;
+                }else{
+                    foreach($output as $line){
+                        $line = str_replace('  ', ' ', $line);
+                        $stats = explode(' ', trim(str_replace($service_name, ' ', $line)));
+                        $result['process'] = $service_name;
+                        $result['cpu']['time'] = $stats[0];
+                        $result['cpu']['percent'] += $stats[1];
+                        $result['mem']['percent'] += $stats[2];
+                        $result['mem']['kb'] += $stats[3];
+                    }
                 }
+                // Adjust cpu % to cpu_count
+                $result['cpu']['percent'] = $result['cpu']['percent'] / $this->cpu_count();
             }
             break;
 
@@ -359,8 +425,9 @@ class Safe_centos_wrapper extends Safe_os_wrapper
         }
         return $return_var;
     }
-    public function set_service_startup($service_name, $is_auto)
+    public function set_service_startup($service_name, $is_auto, $process_name=null)
     {
+        if (isset($process_name)) $service_name = $process_name;
         $rc = null;
         if ($is_auto) {
             safe_sudo_exec('chkconfig --add ' . $service_name, $rc);
@@ -370,18 +437,142 @@ class Safe_centos_wrapper extends Safe_os_wrapper
         }
         return (0 == $rc);
     }
-    public function is_service_auto($service_name)
+    public function is_service_auto($service_name, $process_name=null)
     {
+        if (isset($process_name)) $service_name = $process_name;
         $output = array();
         $output = safe_sudo_exec('chkconfig --list ' . $service_name . ' | grep :on', '', '', $rc, false);
         return (empty($output)) ? false : true;
     }
     public function kill($service_name, $signal, $process_name = null)
     {
+        if (isset($process_name)) $service_name = $process_name;
         $rc = null;
         $result = safe_sudo_exec('/usr/bin/pkill', '-' . $signal . ' ' . $service_name, '', $rc, false);
         return (0 == $rc);
     }
+    public function statistics()
+    {
+        $stats = array();
+
+        // Get /proc/meminfo
+        $output = array();
+        $rc = $this->execute('nohup cat /proc/meminfo', $output);
+        if($rc){
+            /* Sample output
+                root@mtl-nsc-devel-will webconfig
+                cat /proc/meminfo
+                MemTotal:         513340 kB
+                MemFree:           42120 kB
+                Buffers:          108340 kB
+                Cached:           130036 kB
+                ...
+             */
+            $memtotal = explode(" ", preg_replace('/\s+/', ' ', $output[0]));
+            $memfree = explode(" ", preg_replace('/\s+/', ' ', $output[1]));
+            $membuffers = explode(" ", preg_replace('/\s+/', ' ', $output[2]));
+            $memcached = explode(" ", preg_replace('/\s+/', ' ', $output[3]));
+
+            $memtotal = intval(trim($memtotal[1]));
+            $memfree = intval(trim($memfree[1]));
+            $membuffers = intval(trim($membuffers[1]));
+            $memcached = intval(trim($memcached[1]));
+            // In kb
+            $stats['memory']['kb']['total'] = $memtotal;
+            $stats['memory']['kb']['used'] = $stats['memory']['kb']['total'] - $memfree - $membuffers - $memcached;
+            $stats['memory']['kb']['free'] = $memfree + $membuffers + $memcached;
+            // In percent
+            $stats['memory']['percent']['free'] = floatval(($stats['memory']['kb']['free'] / $stats['memory']['kb']['total'])*100.0);
+            $stats['memory']['percent']['used'] = 100.0 - $stats['memory']['percent']['free'];
+        }
+        // Get /proc/meminfo
+        $output = array();
+        $rc = $this->execute('nohup cat /proc/stat', $output);
+        if($rc){
+            /* Sample output
+            root@mtl-nsc-devel-will webconfig
+            cat /proc/stat
+            cpu  1518057 3072 1000613 154507988 64384 0 336660 0 0 0
+            ...
+             */
+            $cpu_info = explode(" ", preg_replace('/\s+/', ' ', $output[0]));
+            unset($cpu_info[0]);
+            // Calculate cpu total
+            $cpu_total = 0;
+            foreach($cpu_info as $k => $v){
+                $v = intval($v);
+                $cpu_info[$k] = $v;
+                $cpu_total += $v;
+            }
+            // Various cpu values: http://linux.die.net/man/5/proc
+            // Latch absolute values
+            $curr_stats['total'] = $cpu_total;
+            // user = user + nice
+            $curr_stats['user'] = $cpu_info[0] + $cpu_info[1];
+            $curr_stats['system'] = $cpu_info[2];
+            $curr_stats['wait'] = $cpu_info[3];
+            $curr_stats['idle'] = $cpu_info[4];
+            // Get previous counters from $_SESSION
+            $prev_stats = array();
+            if($_SESSION['system']['stats']){
+                $prev_stats = $_SESSION['system']['stats'];
+            }
+            // Store stats for next round
+            $_SESSION['system']['stats'] = $curr_stats;
+            // Compute difference since last run
+            $_total = $curr_stats['total'] - $prev_stats['total'];
+            foreach($curr_stats as $k=>$v){
+                $stats['cpu']['percent'][$k] = floatval((($curr_stats[$k]-$prev_stats[$k])/$_total)*100.0);
+            }
+        }
+
+        return $stats;
+    }
+    public function cpu_count()
+    {
+        if($_SESSION['system']['cpu']['count']){
+            return $_SESSION['system']['cpu']['count'];
+        }
+        // Default is 1 CPU at least ;)
+        $cpu_count = 1;
+        $rc = $this->execute('/usr/bin/nproc', $output);
+        if($rc){
+            $cpu_count = intval($output[0]);
+        }
+        $_SESSION['system']['cpu']['count'] = $cpu_count;
+        return $cpu_count;
+    }
+    public function version()
+    {
+        $uname_field = array('kernel-name', 'nodename',  'kernel-release', 'kernel-version', 'machine', 'processor', 'hardware-platform', 'operating-system');
+        $result = array();
+        foreach($uname_field as $item){
+            $output = array();
+            $rc = $this->execute('nohup uname --' . $item, $output);
+            if($rc){
+                $result[$item] = $output[0];
+            }
+        }
+        return $result;
+    }
+    /**
+     * @brief Check if update package can be install
+     *
+     * @param[in out] $reason
+     *
+     * @return true/false
+     */
+    public function can_install_data($info, &$reason)
+    {
+        if('update' == $info['type']){
+            $system_info = $this->version();
+            if ($system_info['hardware-platform'] != $info['architecture']) {
+                $reason[] = 'Cannot install ' . $info['type'] . ' for ' . $info['architecture'] .' on a ' . $system_info['hardware-platform'] . ' platform.';
+                return false;
+            }
+        }
+        return parent::can_install_data($info, $reason);
+    }
 }
 class Safe_debian_wrapper extends Safe_os_wrapper
 {
