@@ -36,7 +36,29 @@ class CI_Form_validation {
 	protected $_error_suffix		= '</p>';
 	protected $error_string			= '';
 	protected $_safe_form_data		= FALSE;
-
+	protected $_validation_obj      = null;// for validation  callback 
+    
+	public function set_validation_obj($obj){
+	    //if the validation have a call back rule, check the callback function in this obj
+	    $this->_validation_obj = $obj;
+	}
+	public function run_rule_callback($method_name, $field_name, $param, $postdata) {
+	    if(method_exists($this -> _validation_obj,$method_name)){
+	        $result = $this -> _validation_obj -> $method_name($field_name, $param, $postdata);
+	        if($result === true){
+	            return $result;
+	        }else{
+	            //$this->safe_form_class->set_message($result);
+	            foreach($result as $key => $message){
+	                $this->set_message($key, $message);
+	            }
+	            return false;
+	        }
+	    }else{
+	        $this->set_message($method_name , $method_name . ' not found! ');
+	        return false;
+	    }
+	}
 	/**
 	 * Constructor
 	 */
@@ -60,7 +82,10 @@ class CI_Form_validation {
 	}
 
 	// --------------------------------------------------------------------
-
+	public function clear_rules()
+	{
+	    $this->_field_data = null;
+	}
 	/**
 	 * Set Rules
 	 *
@@ -165,7 +190,7 @@ class CI_Form_validation {
 	 */
 	public function set_message($lang, $val = '')
 	{
-		if ( ! is_array($lang))
+	    if ( ! is_array($lang))
 		{
 			$lang = array($lang => $val);
 		}
@@ -225,6 +250,22 @@ class CI_Form_validation {
 
 		return $prefix.$this->_field_data[$field]['error'].$suffix;
 	}
+	/**
+	 * Set Error Message
+	 *
+	 * Set the error message associated with a particular field
+	 *
+	 * @access	public
+	 * @param	string	the field name
+	 * @param	string	the error message
+	 * @return	void
+	 */
+	public function set_error($field = '', $message = '')
+	{
+	    if (isset($this->_field_data[$field])){
+	        $this->_field_data[$field]['error'] = sprintf($message, $this->_field_data[$field]['label']);
+	    }
+	}
 
 	// --------------------------------------------------------------------
 
@@ -281,7 +322,9 @@ class CI_Form_validation {
 	 */
 	public function run($group = '')
 	{
-		// Do we even have any data to process?  Mm?
+	    //clear form errors before run again
+	    $this->_error_array = array();
+	    // Do we even have any data to process?  Mm?
 		if (count($_POST) == 0)
 		{
 			return FALSE;
@@ -336,6 +379,11 @@ class CI_Form_validation {
 				if (isset($_POST[$field]) AND $_POST[$field] != "")
 				{
 					$this->_field_data[$field]['postdata'] = $_POST[$field];
+				}else{
+				    if (isset($_FILES[$field]) AND $_FILES[$field] != "")
+				    {
+				        $this->_field_data[$field]['postdata'] = $_FILES[$field];
+				    }
 				}
 			}
 
@@ -474,7 +522,7 @@ class CI_Form_validation {
 	 */
 	protected function _execute($row, $rules, $postdata = NULL, $cycles = 0)
 	{
-		// If the $_POST data is an array we will run a recursive call
+	    // If the $_POST data is an array we will run a recursive call
 		if (is_array($postdata))
 		{
 			foreach ($postdata as $key => $val)
@@ -499,7 +547,6 @@ class CI_Form_validation {
         break;
       }
     }
-
 		if ( ! in_array('required', $rules) AND is_null($postdata))
 		{
 			// Before we bail out, does the rule contain a callback?
@@ -549,7 +596,6 @@ class CI_Form_validation {
 
 			return;
 		}
-
 		// --------------------------------------------------------------------
 
 		// Cycle through each rule and run it
@@ -582,7 +628,7 @@ class CI_Form_validation {
 			$callback = FALSE;
 			if (substr($rule, 0, 9) == 'callback_')
 			{
-				$rule = substr($rule, 9);
+			    $rule = substr($rule, 9);
 				$callback = TRUE;
 			}
 
@@ -594,18 +640,22 @@ class CI_Form_validation {
 				$rule	= $match[1];
 				$param	= $match[2];
 			}
-
 			// Call the function that corresponds to the rule
 			if ($callback === TRUE)
 			{
-				if ( ! method_exists($this->CI, $rule))
+			    if ( ! method_exists($this->CI, $rule))
 				{
-					continue;
+					//run call back rule in the obj
+				    if( $this->_validation_obj != null && method_exists($this->_validation_obj, $rule) ){
+					    $result = $this->run_rule_callback($rule,$row['field'], $param, $postdata);
+					}else{
+				        continue;
+					}
+				}else{
+    				// Run the function and grab the result
+	    			$result = $this->CI->$rule($postdata, $param);
 				}
 
-				// Run the function and grab the result
-				$result = $this->CI->$rule($postdata, $param);
-
 				// Re-assign the result to the master data array
 				if ($_in_array == TRUE)
 				{
@@ -648,8 +698,8 @@ class CI_Form_validation {
 
 					//continue;
 				}else{
-          $result = $this->$rule($postdata, $param);
-        }
+                  $result = $this->$rule($postdata, $param);
+                }
         // If conditionnal required is present and executed rule is required and
         // result is "SKIP" then bail out as condition not meet
         if($rule === "required" AND $conditionnal_required AND $result === "SKIP")
@@ -670,7 +720,7 @@ class CI_Form_validation {
 			// Did the rule test negatively?  If so, grab the error.
 			if ($result === FALSE)
 			{
-				if ( ! isset($this->_error_messages[$rule]))
+			    if ( ! isset($this->_error_messages[$rule]))
 				{
 					if (FALSE === ($line = $this->CI->lang->line($rule)))
 					{
@@ -759,6 +809,9 @@ class CI_Form_validation {
 		{
 			return array_shift($this->_field_data[$field]['postdata']);
 		}
+    if (NULL === $this->_field_data[$field]['postdata']){
+      return $default;
+    }
 
 		return $this->_field_data[$field]['postdata'];
 	}
@@ -903,24 +956,79 @@ class CI_Form_validation {
 	 * @access	public
 	 * @param	string
 	 * @return	bool
+	 * @param  field = field_name,[in|not_in],aaa,bbb,ccc], if $_POST[field_name] in array(aaa,bbb,ccc) field was required
+	 *                 if field='', check it's self value in array(aaa,bbb,ccc) or not
 	 */
 	public function required($str,$field)
 	{
-    $validate = true;
-    if(isset($field) && isset($_POST[$field]))
-      $validate = $_POST[$field];
-
-    if(!$validate || 'false' == $validate)
-      return "SKIP";
-
-		if ( ! is_array($str))
-		{
-			return (trim($str) == '') ? FALSE : TRUE;
-		}
-		else
-		{
-			return ( ! empty($str));
-		}
+	    $validate = true;
+	    if(isset($field)){
+	        if(strpos($field,',') !== false){
+	            $pieces = explode(",", $field);
+	            if(count($pieces) > 2){
+	                $field_name = array_shift($pieces);
+	                $operator = array_shift($pieces);
+	                $data_array = $pieces;
+	                $check_data = '';
+	                if($field_name == ''){
+	                    $check_data = $str;//check self value 
+	                }elseif(isset($_POST[$field_name])){
+	                    $check_data = $_POST[$field_name];
+	                }
+	                $check_result = false;
+	                switch ($operator){
+	                    case 'in':
+	                        $check_result = in_array($check_data,$data_array);
+	                        break;
+	                    case 'not_in':
+	                        $check_result = !in_array($check_data,$data_array);
+	                        break;
+	                }
+	                if($check_result){
+    	                if($field_name == ''){
+    	                    //self value check 
+    	                    return false;
+    	                }elseif(isset($_POST[$field_name])){
+    	                    //need to continue check other rules
+    	                   $validate = 'true';
+    	                }
+	                }else{
+	                    if($field_name == ''){
+	                        //self value not empty, but need to continue check other rules
+	                        $validate = 'true';
+	                    }else{
+	                        //pass by other rules
+	                        $validate = 'false';
+	                    }
+	                }
+	            }else{
+	                $validate = 'false';
+	            }
+	        }else{
+	            if(isset($_POST[$field])){
+	                $validate = $_POST[$field];
+	            }else{
+	                $validate = 'false';
+	            }
+	        }
+	    }
+	    
+    
+        if(!$validate || 'false' == $validate)
+          return "SKIP";
+    
+    		if ( ! is_array($str))
+    		{
+    			return (trim($str) == '') ? FALSE : TRUE;
+    		}
+    		else
+    		{
+    			return ( ! empty($str));
+    		}
+	}
+	public function disabled($str)
+	{
+	    return false;
 	}
 
 	// --------------------------------------------------------------------
@@ -1198,7 +1306,7 @@ class CI_Form_validation {
 	 */
 	public function alpha_dash($str)
 	{
-		return ( ! preg_match("/^([-a-z.0-9_-])+$/i", $str)) ? FALSE : TRUE;
+	    return ( ! preg_match("/^([-a-z.0-9_-])+$/i", $str)) ? FALSE : TRUE;
 	}
 
 	// --------------------------------------------------------------------
@@ -1473,13 +1581,33 @@ class CI_Form_validation {
 	    }
 	    return FALSE;
 	}
+	public function ssl_ip_or_domain($str)
+	{
+	    // first try to validate ip
+	    if(strtolower(substr($str, 0, 6)) == 'ssl://'){
+	        $token = substr($str, 7);
+        }else{
+            $token = $str;
+        }
+	    return $this->valid_ip_or_domain($token);
+	}
   public function valid_sip_user($str){
     return ( ! preg_match("/^([-a-z.0-9_-i\+\&=\!~\$\?\(\)\*\'\,\;\/])+$/i", $str)) ? FALSE : TRUE;
   }
 
 	
 	// --------------------------------------------------------------------
-	
+    /**
+    * Test if string is valid PORT number.
+    * @param string $str
+    */
+    public  function valid_port_number($str) {
+          if(ctype_digit($str) && intval($str) > 0 && intval($str) <= 65535){
+              return true;
+          }else{
+              return false;
+          }
+    }
 	/**
 	 * Test if string has a valid ip, domain name or IP:PORT notation.
 	 * @param string $str
@@ -1488,13 +1616,14 @@ class CI_Form_validation {
 		$pos = strpos($str,':');
 		if ($pos !== false) {
 			$ip_port = explode(':',$str);
-			if (count($ip_port) == 2 && is_numeric($ip_port[1]) && $this->valid_ip_or_domain($ip_port[0]) &&  ($ip_port[1] > 0 || $ip_port < 65536))
-				return true;
-			else
-				return false;
+			if (count($ip_port) == 2 &&  $this->valid_ip_or_domain($ip_port[0]) &&  $this->valid_port_number($ip_port[1])){
+			    return true;
+			}else{
+			    return false;
+			}
+		}else{
+		    return $this->valid_ip_or_domain($str);
 		}
-		else
-			return $this->valid_ip_or_domain($str);
 	}
 	
 	// --------------------------------------------------------------------
@@ -1508,7 +1637,7 @@ class CI_Form_validation {
 		$success_count = 0;
 		$fail_count = 0;
 		
-		if (trim($str) == "NULL")
+		if (trim($str) == "")
 			return true;
 		else {
 			if (strpos($str,',') !== false) {
@@ -1625,7 +1754,7 @@ class CI_Form_validation {
 	 */
 	
 	public function sip_fail_response_code ($str) {
-		if (is_numeric($str) && preg_match('/^[56][0-9][0-9]/i',$str))
+		if (is_numeric($str) && preg_match('/^[56][0-9][0-9]$/i',$str))
 			return true;
 		
 		return false;
@@ -1748,14 +1877,27 @@ class CI_Form_validation {
     * @return 
    */
   public function in_list($str, $list) {
-    if(0 != strlen($str)){
+      //if $str is array, need to check each item of $str is in $list
+      if(!is_array($str)){
+          $str = array($str);
+      }
       $options = explode(',',$list);
-      return (in_array($str,$options));
-    }
-    return FALSE;
+      foreach($str as $key => $tmp_item){
+          if(0 == strlen($tmp_item) || !in_array($tmp_item,$options)) {
+              return false;
+          }
+      }
+      return true;
   }
-
-
+  
+  public function not_in_list($str, $list) {
+      if(0 != strlen($str)){
+          $options = explode(',',$list);
+          return (!in_array($str,$options));
+      }
+      return true;
+  }
+  
 	// --------------------------------------------------------------------
 
 	/**
