@@ -62,13 +62,52 @@ class Sng_archive_manager extends Sng_data_manager
             // Populate generation result table
             $data['table_title'] = 'Archive - ' . $data_key;
             $table = new Safe_table_class();
-            $table->data = $reason['notif_error'];
+            foreach($reason as $item){
+                $fix_url = '';
+                if(isset($item['url'])){
+                    $fix_url = $item['url'];
+                }
+                $module = $this->_the_app->find_module_by_name($item['module']);
+                if($module){
+                    $mod_name = $module->description();
+                    $obj_def = $module->get_aggregate_object_definition($item['obj_type']);
+                    if($obj_def){
+                        $obj_type = $obj_def['name'];
+                        if($fix_url == ''){
+                            $fix_url = $obj_def['controller_url'];
+                        }
+                    }else{
+                        $obj_type = $item['obj_type'];
+                    }
+                    $obj_name = $item['obj_name'];
+                }else{
+                    $mod_name = $item['module'];
+                    $obj_type = $item['obj_type'];
+                    $obj_name = $item['obj_name'];
+                }
+                $table->data[] = array(
+                        ' '. WEBCONFIG_ICON_XMARK,
+                        $mod_name,
+                        $obj_type,
+                        $obj_name,
+                        $item['description'],
+                );
+                if($fix_url){
+                    $table->line_button[] = array(
+                            array(
+                                    'url' => $fix_url,
+                                    'name' => 'Configure',
+                                    'label' => ''
+                            )
+                    );
+                }
+            }
             $table->footer_button = array(
-                array(
-                    'url' => $back_url,
-                    'name' => 'Back'
-                )
-            );
+                     array(
+                             'url' => $back_url,
+                             'name' => 'Back'
+                             )
+                    );
             $data['table'] = $table->generate();
         }else{
             $info = $archive_mgr->info($data_key);
