@@ -401,6 +401,10 @@ class Sng_config_manager extends Safe_CI_Controller
         }
         // Aggregate object tables
         if($this->display_aggregate_object){
+            $child_base_url = $base_url;
+            if($obj_type && $obj_name){
+                $child_base_url = $child_base_url . '/' . $obj_type.'/' . $obj_name;
+            }
             foreach ($module->has_aggregate_objects(false, true) as $_type => $_definition) {
                 if($_definition['configurable'] === false){
                     continue;
@@ -425,12 +429,9 @@ class Sng_config_manager extends Safe_CI_Controller
                 $data['table_title'] = $table->set_title($_definition['name']);
                 $data['table_id'] = $_id.'_'.$_type;
                 $table->set_controller_url($this->router->fetch_class());
-                if($obj_type && $obj_name){
-                    $base_url = $base_url . '/' . $obj_type.'/' . $obj_name;
-                }
 
                 $table_heading_array = array();
-                $this->generate_table_data($table, $data, $module, $_type, $base_url, $custom, $table_heading_array);
+                $this->generate_table_data($table, $data, $module, $_type, $child_base_url, $custom, $table_heading_array);
 
                 $table->set_headings($table_heading_array);
                 if(isset($custom['allow_create_button'])){
